﻿//●波罫.jsx
//　波罫をグラフィックラインを用いて作成し、アンカー付きオブジェクトとして配置するスクリプトです。
//　波罫を引く範囲を選択して実行してください。ただし、選択範囲は行をまたがないようにして下さい。
//　波罫の大きさ（波の周期）は選択範囲の最初の文字の級数を基準にしています。
//　一時的にライブラリを使用しますので、ライブラリをひとつ開いておいてください。
//　開いていない場合には、このスクリプトの入っているフォルダ内のtemp.indlを開きます。
//　アンカーは選択範囲の最初の文字の次に挿入されます。（行頭から始まる文字列に波罫を引くときに、波罫が前の行の行末に残るのを防ぐため）
//　波罫の長さ、およびXオフセットは選択範囲の位置を測って計算していますので、文字組の影響で微妙にずれる場合があります。
//　長過ぎる場合にはダイレクト選択ツールで波罫のコントロールポイントを選択し、削除して縮めてください。
//　波罫の引き初めの位置がずれている場合には、InDesignの[オブジェクト]メニューの[アンカー付きオブジェクト]のXオフセットの値をいじってください。
//　2006.11.6に表組内でも波罫が引けるように修正しました。しかし、横組ストーリー中の表組のセルで縦組の部分、縦組ストーリー中の表組のセルで横組の部分では、アンカー付きオブジェクトが正常に表示されないため使用出来ません。
//										2006.12.08/13:30 修正	(C)CopyRight Mishima Print Co.
//
//	1.このスクリプトを使用して何らかの損害が生じたとしても、責任は負いかねますので、自己責任で使用して下さい。
//	2.このスクリプトを使い易いように、改変、改良をして同一部署ないで使用するのは構いません。
//	3.このスクリプト及び、改変、改良を加えたスクリプトを無償で再配布する場合にはご連絡ください。
//	4.このスクリプト及び、改変、改良を加えたスクリプトを有償で再配布する事は禁止します。
//
//						2006.12.05　三島梅花藻	/ Mishima Print Co.
//						e-mail：tuchiya☆mishima-print.co.jp	（☆を@に置き換えて下さい。）
						
TATEGAKI = 1986359924;
SentoMojiStyleSyuryou = 1396855379;

if (app.documents.length != 0){
	if (app.selection.length == 1){
		switch (app.selection[0].constructor.name){
			case "Text":
			case "Character":
			case "Word":
			case "Line":
			case "TextStyleRange":
			case "Paragraph":
			case "TextColumn":
				myDisplayDialog();
				break;
			default:
				alert("選択された部分はテキストではありません。テキストを範囲指定してください。");
				break;
		}
	}
	else{
		alert("テキスト部分を選択し直してください。");
	}
}
else{
	alert("No documents are open. Please open a document, select some text, and try again.");
}

function myDisplayDialog(){
	var myDoc = app.activeDocument;
	var zoom = app.activeWindow.zoomPercentage;
	with (myDoc.viewPreferences){ 					//計測単位の退避と設定
		var myOldXUnits = horizontalMeasurementUnits; 
		var myOldYUnits = verticalMeasurementUnits;
		var myOldTextUnits = textSizeMeasurementUnits;
		var myOldLineUnits = lineMeasurementUnits;
		var myOldTypographUnits = typographicMeasurementUnits;
		horizontalMeasurementUnits = MeasurementUnits.millimeters; 
		verticalMeasurementUnits = MeasurementUnits.millimeters;
		textSizeMeasurementUnits = MeasurementUnits.q;
		lineMeasurementUnits = MeasurementUnits.millimeters;
		typographicMeasurementUnits = MeasurementUnits.ha;
	}
	var mySwatches = app.activeDocument.swatches;
	var SwatcheList = Array();
	for (var i=0;i<mySwatches.length;i++)	SwatcheList[i]=mySwatches[i].name;
	var myFontSize = app.selection[0].characters[0].pointSize;
	var mFontSize = eval(myFontSize);
	var waveoff = mFontSize/2 + 1;								//波罫を引く位置の初期値
	var myStory = app.selection[0].parentStory;
	var StartPtr = app.selection[0].characters.firstItem().index;
	var TerminalPtr = app.selection[0].characters.lastItem().index;
	var tchar = app.selection[0].characters.lastItem().contents;	//選択範囲の最後の文字
	var tategakiFlag = false;
	var cellFlag = false;
	if (myStory.storyPreferences.storyOrientation==TATEGAKI) tategakiFlag = true ;	//縦書き
	if (app.selection[0].parent.constructor.name=="Cell"){		// 表組内
		cellFlag = true;
		myStory = app.selection[0].parent;
		if (app.selection[0].parent.writingDirection==TATEGAKI) tategakiFlag = true;	//セル内縦組
		else tategakiFlag = false;
	}
	widthLst = ["0.12","0.25","0.4","0.5"];	//線幅のリスト
	waveoffLst = [""+(mFontSize/2+1),""+(mFontSize/2+2),""+(mFontSize+1)]	//オフセットのリスト
	var myDialog;
	with(myDialog = app.dialogs.add({name:"波罫引きスクリプト"})){
		with(dialogColumns.add()){
			with(borderPanels.add()){
				staticTexts.add({staticLabel:"仮想ボディの中央からの距離(歯):"});
				with(dialogColumns.add()){
					wavelineOffset = realComboboxes.add({editValue:waveoff,minimumValue:-400,maximumValue:400,smallNudge:1,stringList:waveoffLst});
				}
			}
			with(borderPanels.add()){
				staticTexts.add({staticLabel:"線の幅		(mm):"});
				with(dialogColumns.add()){
					wavelineWeight = realComboboxes.add({editValue:0.25,minimumValue:0,maximumValue:10,smallNudge:0.1,stringList:widthLst});
				}
			}
			with(borderPanels.add()){
				staticTexts.add({staticLabel:"カラー　　　　:"});
				with(dialogColumns.add()){
					lineSwatch = dropdowns.add({stringList:SwatcheList, selectedIndex:2});
				}
			}
		}
	}
	if (myDialog.show()) {
		var stleadingAki = app.selection[0].characters[0].leadingAki;
		app.selection[0].characters[0].leadingAki = 0;
		if (tategakiFlag){		//縦書き
			if (StartPtr==0 || (myStory.characters[StartPtr-1].baseline>myStory.characters[StartPtr].baseline)){
				if (cellFlag){
					myStory.lines[0].insertionPoints[0].contents = SentoMojiStyleSyuryou;
					var Y0 = myStory.characters[0].baseline;
					myStory.characters[0].remove();
				} else{
					var Y0 = app.selection[0].characters[0].parentTextFrames[0].geometricBounds[0]+app.selection[0].characters[0].leftIndent;
				}
			} else {
				var Y0 = myStory.characters[StartPtr-1].baseline+mFontSize/4*0.12;
			}
			var Yt = myStory.characters[TerminalPtr].baseline+myStory.characters[TerminalPtr].pointSize/4*0.12;
			if (tchar=="。" || tchar=="、" || tchar=="」") Yt = Yt - mFontSize/8 ;
			var len = Yt - Y0;
			if (len <= 0){
				if (cellFlag){
					myStory.lines[0].insertionPoints[0].contents = SentoMojiStyleSyuryou;
					var Y0 = myStory.characters[0].baseline;
					myStory.characters[0].remove();
				} else{
					Yt = app.selection[0].characters[0].parentTextFrames[0].geometricBounds[2]-app.selection[0].characters[0].rightIndent;
					len = Yt - Y0;
				}
			}
			if (len > 0){
				pNumber = Math.round(len*16/mFontSize);
				with (lineObj = app.activeDocument.graphicLines.add()){	//波罫をグラフィックラインで作成
					strokeColor = mySwatches[lineSwatch.selectedIndex].name;
					strokeType = "ベタ";
					strokeWeight = wavelineWeight.editValue;
					fillColor = "None";
					with (paths.item(0)){
						while(pathPoints[-1].index<pNumber)	pathPoints.add();
						if (wavelineOffset.editValue>0){
							var x0 = myStory.characters[StartPtr].insertionPoints[0].horizontalOffset+(wavelineOffset.editValue/4+mFontSize/8+mFontSize/64);
						} else if (wavelineOffset.editValue<0){
							var x0 = myStory.characters[StartPtr].insertionPoints[0].horizontalOffset+(wavelineOffset.editValue/4+mFontSize/8-mFontSize/64);
						} else {
							var x0 = myStory.characters[StartPtr].insertionPoints[0].horizontalOffset+(wavelineOffset.editValue/4+mFontSize/8);
						}
						var xw = mFontSize/48;
						var y0 = myStory.characters[StartPtr].baseline-mFontSize/4*0.88;
						var dy = mFontSize/16;
						var yw = mFontSize/48;
						for (var i=0;i<pNumber+1;i++){
							with (pathPoints[i]){
								anchor = Array(x0+"mm",y0+dy*i+"mm");
								if (i%2==0){
									leftDirection = Array(x0+xw+"mm",y0+dy*i-yw+"mm");
									rightDirection = Array(x0-xw+"mm",y0+dy*i+yw+"mm");
								} else {
									leftDirection = Array(x0-xw+"mm",y0+dy*i-yw+"mm");
									rightDirection = Array(x0+xw+"mm",y0+dy*i+yw+"mm");
								}
								pointType = PointType.smooth;
							}
						}
					}
				}
				app.selection = myStory.characters[StartPtr].insertionPoints[1];
				var myLibraries = app.libraries;	//一時的にライブラリにlineObjを登録するためライブラリを開く
				var myScriptFileName = app.activeScript;
				var myScriptFile = File(myScriptFileName);
				var myFolder = myScriptFile.parent;
				if (myLibraries.length != 0){
					myLib = myLibraries.firstItem();
				} else {
					var fname = myFolder.getFiles("temp.indl")[0];
					myLib = myLibraries.add(fname);
				}
				var myAset = lineObj.store(myLib);				//ライブラリに保存
				app.selection = lineObj;
				with (app.selection[0]){				//アンカー付きオブジェクトにする波罫のラインオブジェクトの左上の位置を記憶
					var mylineObjPosition = new Array(geometricBounds[0],geometricBounds[1]);
				}
				lineObj.remove();
				app.selection = myStory.characters[StartPtr].insertionPoints[1];
				var myObjs = myAset.placeAsset(app.activeDocument);		//ドキュメント内に配置
				myAset.remove();
				with (mylineObj = myObjs[0]){
					app.activeWindow.zoomPercentage = zoom;
					try{mylineObj.rotationAngle = 0;}
					catch(e){alert(e);mylineObj.rotationAngle = 0;}
					with (anchoredObjectSettings){
						anchoredPosition = AnchorPosition.anchored;
						anchorPoint = AnchorPoint.topCenterAnchor;
						anchorXoffset = 0;
						anchorYoffset = 0;
						horizontalAlignment = HorizontalAlignment.centerAlign;
						horizontalReferencePoint = AnchoredRelativeTo.anchorLocation;
						verticalAlignment = VerticalAlignment.topAlign;
						verticalReferencePoint = VerticallyRelativeTo.lineBaseline;
						pinPosition = false;
					}
				}
				app.activeWindow.zoomPercentage = zoom;
				with (mylineObj){								//アンカー付きオブジェクト化した波罫のラインオブジェクトの位置
					var newlineObjPosition = new Array(geometricBounds[0],geometricBounds[1]);
				}
				with (mylineObj.anchoredObjectSettings){		//アンカー付きオブジェクトのオフセットを計算
					anchorXoffset =	- (mylineObjPosition[0] - newlineObjPosition[0]) / 25.4 * 72;
					anchorYoffset = - (mylineObjPosition[1] - newlineObjPosition[1]) / 25.4 * 72;
//					lockPosition = true;					//間違って動かしてしまわないようにロックする
				}
			} else {
				alert("波罫を引く範囲は1行ごとに指定して下さい。");
			}
		} else {			//横書き
			var X0 = myStory.characters[StartPtr].insertionPoints[0].horizontalOffset;
			var Xt = myStory.characters[TerminalPtr].insertionPoints[1].horizontalOffset;
			if (tchar=="。" || tchar=="、" || tchar=="」") Xt = Xt - mFontSize/8 ;
			var len = Xt - X0;
			if (len <= 0){
				if (cellFlag){
					myStory.characters[TerminalPtr+1].insertionPoints[0].contents = SentoMojiStyleSyuryou;
					var Xt = myStory.characters[TerminalPtr+1].horizontalOffset;
					myStory.characters[TerminalPtr+1].remove();
					len = Xt - X0;
				} else{
					Xt = app.selection[0].characters[0].parentTextFrames[0].geometricBounds[3]-app.selection[0].characters[0].rightIndent;
					len = Xt - X0;
				}
			}
			if (len > 0){
				pNumber = Math.round(len*16/mFontSize);
				with (lineObj = app.activeDocument.graphicLines.add()){
					strokeColor = mySwatches[lineSwatch.selectedIndex].name;
					strokeType = "ベタ";
					strokeWeight = wavelineWeight.editValue;
					fillColor = "None";
					with (paths.item(0)){
						while(pathPoints[-1].index<pNumber)	pathPoints.add();
						var x0 = X0;
						var dx = mFontSize/16;
						var xw = mFontSize/48;
						if (wavelineOffset.editValue>0){
							var y0 = myStory.characters[StartPtr].insertionPoints[0].baseline-mFontSize/4*0.38+(wavelineOffset.editValue/4+mFontSize/64);
						} else if (wavelineOffset.editValue<0){
							var y0 = myStory.characters[StartPtr].insertionPoints[0].baseline-mFontSize/4*0.38+(wavelineOffset.editValue/4-mFontSize/64);
						} else {
							var y0 = myStory.characters[StartPtr].insertionPoints[0].baseline-mFontSize/4*0.38+(wavelineOffset.editValue/4);
						}
						var yw = mFontSize/48;
						for (var i=0;i<pNumber+1;i++){
							with (pathPoints[i]){
								anchor = Array(x0+dx*i+"mm",y0+"mm");
								if (i%2==0){
									leftDirection = Array(x0+dx*i-xw+"mm",y0+yw+"mm");
									rightDirection = Array(x0+dx*i+xw+"mm",y0-yw+"mm");
								} else {
									leftDirection = Array(x0+dx*i-xw+"mm",y0-yw+"mm");
									rightDirection = Array(x0+dx*i+xw+"mm",y0+yw+"mm");
								}
								pointType = PointType.smooth;
							}
						}
					}
				}
				app.selection = myStory.characters[StartPtr].insertionPoints[1];
				var myLibraries = app.libraries;	//一時的にライブラリにlineObjを登録するためライブラリを開く
				var myScriptFileName = app.activeScript;
				var myScriptFile = File(myScriptFileName);
				var myFolder = myScriptFile.parent;
				if (myLibraries.length != 0){
					myLib = myLibraries.firstItem();
				} else {
					var fname = myFolder.getFiles("temp.indl")[0];
					myLib = myLibraries.add(fname);
				}
				var myAset = lineObj.store(myLib);				//ライブラリに保存
				app.selection = lineObj;
				with (app.selection[0]){				//アンカー付きオブジェクトにする波罫のラインオブジェクトの左上の位置を記憶
					var mylineObjPosition = new Array(geometricBounds[0],geometricBounds[1]);
				}
				lineObj.remove();
				app.selection = myStory.characters[StartPtr].insertionPoints[1];
				var myObjs = myAset.placeAsset(app.activeDocument);		//ドキュメント内に配置
				myAset.remove();
				with (mylineObj = myObjs[0]){
					with (anchoredObjectSettings){
						anchoredPosition = AnchorPosition.anchored;
						anchorPoint = AnchorPoint.leftCenterAnchor;
						anchorYoffset = 0;
						anchorXoffset = 0;
						horizontalAlignment = HorizontalAlignment.leftAlign;
						horizontalReferencePoint = AnchoredRelativeTo.anchorLocation;
						verticalAlignment = VerticalAlignment.centerAlign;
						verticalReferencePoint = VerticallyRelativeTo.lineBaseline;
						pinPosition = false;
					}
				}
				app.activeWindow.zoomPercentage = zoom;
				with (mylineObj){								//アンカー付きオブジェクト化した波罫のラインオブジェクトの位置
					var newlineObjPosition = new Array(geometricBounds[0],geometricBounds[1]);
				}
				with (mylineObj.anchoredObjectSettings){		//アンカー付きオブジェクトのオフセットを計算
					anchorYoffset =	(mylineObjPosition[0] - newlineObjPosition[0]) / 25.4 * 72;
					anchorXoffset = - (mylineObjPosition[1] - newlineObjPosition[1]) / 25.4 * 72;
//					lockPosition = true;					//間違って動かしてしまわないようにロックする
				}
			} else {
				alert("波罫を引く範囲は1行ごとに指定して下さい。");
			}
		}
		myStory.characters[StartPtr].leadingAki = stleadingAki;
	}
	myDialog.destroy();
	with (myDoc.viewPreferences){ 
		try{ 
			horizontalMeasurementUnits = myOldXUnits; 
			verticalMeasurementUnits = myOldYUnits; 
			textSizeMeasurementUnits = myOldTextUnits;
			lineMeasurementUnits = myOldLineUnits;
			typographicMeasurementUnits = myOldTypographUnits;
		} 
		catch(myError){ 
			alert("Could not reset custom measurement units."); 
		}
	}
}